=== modified file 'lib/lp/registry/model/productseries.py'
--- lib/lp/registry/model/productseries.py	2011-05-16 18:09:12 +0000
+++ lib/lp/registry/model/productseries.py	2011-05-16 19:52:01 +0000
@@ -40,6 +40,9 @@
     SQLBase,
     sqlvalues,
     )
+from canonical.launchpad.components.decoratedresultset import (
+    DecoratedResultSet,
+    )
 from canonical.launchpad.interfaces.launchpad import ILaunchpadCelebrities
 from canonical.launchpad.webapp.publisher import canonical_url
 from canonical.launchpad.webapp.sorting import sorted_dotted_numbers
@@ -203,11 +206,19 @@
     def releases(self):
         """See `IProductSeries`."""
         store = Store.of(self)
+
+        # The Milestone is cached too because most uses of a ProductRelease
+        # need it. The decorated resultset returns just the ProductRelease.
+        def decorate(row):
+            product_release, milestone = row
+            return product_release
+
         result = store.find(
-            ProductRelease,
-            And(Milestone.productseries == self,
-                ProductRelease.milestone == Milestone.id))
-        return result.order_by(Desc('datereleased'))
+            (ProductRelease, Milestone),
+            Milestone.productseries == self,
+            ProductRelease.milestone == Milestone.id)
+        result = result.order_by(Desc('datereleased'))
+        return DecoratedResultSet(result, decorate)
 
     @property
     def release_files(self):

=== modified file 'lib/lp/registry/tests/test_productseries.py'
--- lib/lp/registry/tests/test_productseries.py	2011-05-16 18:09:12 +0000
+++ lib/lp/registry/tests/test_productseries.py	2011-05-16 19:52:01 +0000
@@ -10,6 +10,7 @@
 from zope.security.proxy import removeSecurityProxy
 
 from canonical.launchpad.ftests import login
+from canonical.launchpad.interfaces.lpstorm import IStore
 from canonical.testing.layers import (
     DatabaseFunctionalLayer,
     LaunchpadFunctionalLayer,
@@ -32,6 +33,34 @@
     )
 
 
+class ProductSeriesReleasesTestCase(TestCaseWithFactory):
+    """Test for ProductSeries.release property."""
+
+    layer = DatabaseFunctionalLayer
+
+    def test_releases(self):
+        # The release property returns an iterator of releases ordered
+        # by date_released from youngest to oldest.
+        series = self.factory.makeProductSeries()
+        milestone = self.factory.makeMilestone(
+            name='0.0.1', productseries=series)
+        release_1 = self.factory.makeProductRelease(milestone=milestone)
+        milestone = self.factory.makeMilestone(
+            name='0.0.2', productseries=series)
+        release_2 = self.factory.makeProductRelease(milestone=milestone)
+        self.assertEqual(
+            [release_2, release_1], list(series.releases))
+
+    def test_releases_caches_milestone(self):
+        # The release's milestone was cached when the release was retrieved.
+        milestone = self.factory.makeMilestone(name='0.0.1')
+        self.factory.makeProductRelease(milestone=milestone)
+        series = milestone.series_target
+        IStore(series).invalidate()
+        [release] = [release for release in series.releases]
+        self.assertStatementCount(0, getattr, release, 'milestone')
+
+
 class ProductSeriesGetReleaseTestCase(TestCaseWithFactory):
     """Test for ProductSeries.getRelease()."""
 

